==> ciao-4.6-src-core.tar.gz: ciao-4.6/src/lib/datamodel/afkernel/afcoord.c <==
/*                                                                
**  Copyright (C) 2003-2009  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/**********************************************************************/
/*                                                                    */
/*  afcoord.c                                                         */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/**********************************************************************/

#include "afkernelP.h"
#include "../ftkernel/ftkernelP.h"

dmBool afp_radesys_equinox_validation(dmBlock *pBlock, long colNumber );

/***********************************************************************/
/*                                                                     */
/*   Function: afp_coord_write_mtype()                                 */
/*                                                                     */
/*   Description:                                                      */
/*              taken from ftkernel                                    */
/*                                                                     */
/*  Input parameters:                                                  */
/*                    dmBlock       *block                             */
/*                    dmDescriptor  *dd                                */
/*                                                                     */
/*  Output parameters:                                                 */
/*                  none                                               */
/*                                                                     */
/*  Returns:                                                           */
/*           void                                                      */
/*                                                                     */
/***********************************************************************/
void afp_coord_write_mtype( dmBlock *block, dmDescriptor *dd )
{
  afBlock *ablock = NULL;

  char  mform[CARD_SIZE];
  char  mtype[CARD_SIZE];
  char  keyname[CARD_SIZE];
  char *value = NULL;
  char *ptr   = NULL;
  char *unit  = NULL; 

  long n     = 0;
  long dim   = 0;
  long cptno = 0;


  dim = dmi_descriptor_get_element_dim( dd );
  cs_copy( dmi_descriptor_get_name( dd ), mtype );  

  ablock = (afBlock*)dmi_block_get_kb( block);

  /* Check if this MTYPE already exists.. */
  n = dmu_keylist_check_index( ablock->keylist, "MTYPE", mtype );
  
  if ( n == 0 )
  {
    /* There are no keywords with this for this system.. create them */
    n = dmu_keyword_max_index( ablock->keylist, "MFORM" );
    ++n;

    /* Write MTYPE keyword */
    sprintf( keyname, "MTYPE%ld", n );
    value = &mtype[0];

    afp_key_write( block, 0, keyname, DT_CHAR, &value, 1,
 		                      NULL, "DM Keyword: Descriptor name." );

    /* Write MFORM keyword */
    sprintf( keyname, "MFORM%ld", n );
    cptno = 1;
    ptr = mform;
    ptr += sprintf( ptr, "%s", dmi_descriptor_get_cpt_name( dd, cptno ) );

    for ( cptno = 2 ; cptno <= dim; cptno++ )
      ptr += sprintf( ptr, ",%s", dmi_descriptor_get_cpt_name( dd, cptno ));

    value = &mform[0];
    unit = dmi_descriptor_get_unit( dd );

    if ( cs_is_blank(unit))
    {
      /* Back compat for regression */
      afp_key_write( block, 0, keyname, DT_CHAR, &value, 1,
		     unit, "DM Keyword: Descriptor value." );
    }
    else
    {
      afp_key_write( block, 0, keyname, DT_CHAR, &value, 1, unit, NULL );
    }
  }
}



/***********************************************************************/
/*                                                                     */
/*  Function: afp_scalar_coord_write()                                 */
/*                                                                     */
/*  Description:   taken from ftkernel                                 */
/*                                                                     */
/*  Input parameters:      dmDescriptor   *dd                          */
/*                         double          crpix                       */
/*                         double          crval                       */
/*                         double          cdelt                       */
/*                         char           *ctype                       */
/*                         long            cptno                       */
/*                         int             is_physical                 */
/*                         int             ok                          */
/*                                                                     */
/*  Output parameters:                                                 */
/*           none                                                      */
/*                                                                     */
/*  Returns:                                                           */
/*           void                                                      */
/*                                                                     */
/***********************************************************************/
void afp_scalar_coord_write( dmDescriptor *dd, double crpix, double crval,
                             double cdelt, char *ctype, long cptno,
                             int is_physical, int ok )
{
  ftCoord *ftcoord = NULL;

  dmBlock          *block     = NULL;
  dmCoordTransform *transform = NULL;
  dmBlockType       type;
  dmBool            bIsICRS   = dmFALSE;

  char   keyname[CARD_SIZE];
  char  *wcstyp = "PHYSICAL";
  char  *unitp  = NULL;
  char  *name   = NULL;
  char  *desc   = NULL;
  char  *unit   = NULL;

  double equinox  = 0.0;
  double poles[2] = { 0.0, 0.0};
  double theta    = 0.0;

  long axisno   = 0;
  long no       = 0;
  long subno    = 0;
  long flag     = 0;
  int  is_image = 0;
  int  axis     = 0;


  if ( !dd )
    return;

  block     = dmi_descriptor_get_block( dd );
  type      = dmi_block_get_raw_type( block );
  ftcoord   = dmi_descriptor_get_kdd( dd );
  transform = dmi_descriptor_get_aux( dd );
  subno     = dmi_coord_get_transform_no( dd );
  no       = dmi_coord_get_scalar_col_no( dd );
  axisno   = dmi_descriptor_get_group_axis_no( dd );
  flag     = dmi_descriptor_get_flag( dd );

  if ( !is_physical )
    is_physical = ( flag & DM_FLAG_PHYS ) || ( ftcoord && ftcoord->sub == 'P' );

  is_image = ( type == dmIMAGE );

  if ( !ftcoord )
  {
    ftcoord = ftp_coord_alloc();
    dmi_descriptor_set_kdd( dd, (void*)ftcoord );

    if ( is_physical )
      ftcoord->sub = 'P';
    else if ( subno > 1 )
      ftcoord->sub =  'A' + subno - 2;
    else
      ftcoord->sub = ' ';

    ftcoord->axis = axisno;

    if ( is_image && axisno > 0 )
      ftcoord->no = 1;
    else
      ftcoord->no = no;
  }

  axis = ( ftcoord && ftcoord->axis > 0 );

  ftp_format_ctype( "CTYPE", "TCTYP", "TCTYP", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );

  desc  = dmi_descriptor_get_desc( dd );
  unit  = dmi_descriptor_get_unit( dd ); 
  unitp = unit;

  if ( dm_global & DM_GLOBAL_OLD )
    unitp = NULL;

  afp_key_write( block, 0, keyname, DT_CHAR, &ctype, 1, unitp, desc );

  ftp_format_ctype( "CRVAL", "TCRVL", "TCRVL", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
  afp_key_write( block, 0, keyname, DT_REAL8, &crval, 1, NULL, NULL );
  ftp_format_ctype( "CRPIX", "TCRPX", "TCRPX", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
  afp_key_write( block, 0, keyname, DT_REAL8, &crpix, 1, NULL, NULL );
  ftp_format_ctype( "CDELT", "TCDLT", "TCDLT", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
  afp_key_write( block, 0, keyname, DT_REAL8, &cdelt, 1, NULL, NULL );

  if ( !ok )
  {
    // FITS Standard Compliance  July 2010
    //  Use TCNA on output instead of TCNAM

    ftp_format_ctype( "CNAME", "TCNA", "TCNA", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );

    name = dmi_descriptor_get_name( dd );
    afp_key_write( block, 0, keyname, DT_CHAR, &name, 1, NULL, NULL );
  }

  if ( is_physical && is_image )
  {
    ftp_format_ctype( "WCSTY", "WCSTY", "WCSTY", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
    afp_key_write( block, 0, keyname, DT_CHAR, &wcstyp, 1, NULL, NULL );
  }

  if ( unit && *unit )
  {
    ftp_format_ctype( "CUNIT", "TCUNI", "TCUNI", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
    ok = ! ( ( dm_global & DM_GLOBAL_OLD ) && is_physical && cs_eq( unit, "pixel" ));

    if ( ok )
      afp_key_write( block, 0, keyname, DT_CHAR, &unit, 1, NULL, NULL );
  }

  /* IRAF and DS9 compatibility */
  if ( is_image && is_physical && cdelt != 0.0 )
  {
    double ltv = 0.0;
    double ltm = 0.0;

    ltv = crpix - crval / cdelt;
    ltm = 1.0 / cdelt;

    sprintf( keyname, "LTV%ld", ftcoord->axis );  
    afp_key_write( block, 0, keyname, DT_REAL8, &ltv, 1, NULL, NULL );  
    sprintf( keyname, "LTM%ld_%ld", ftcoord->axis, ftcoord->axis );  
    afp_key_write( block, 0, keyname, DT_REAL8, &ltm, 1, NULL, NULL );  
  }


  if ( cptno == 1 )
  {
    equinox = dmi_coord_get_equinox( transform );

    // Code Change  FITS Standards  September 15, 2010
    // Need to check value of RADESYS in order to verify EQUINOX valaue

    bIsICRS = afp_radesys_equinox_validation( block, ftcoord->no );


    if (  !bIsICRS && equinox != 0 )
    {
      ftp_format_ctype( "EQUINOX", "EQUI", "EQUI", ftcoord->no, -1, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &equinox, 1, NULL, NULL );
    }

    dmi_coord_get_poles( transform, poles );

    if ( poles[0] != 0.0 )
    {
      ftp_format_ctype( "LONGPOLE", "LONP", "LONP", ftcoord->no, -1, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &poles[0], 1, NULL, NULL );
    }

    if ( poles[1] != 0.0 )
    {
      ftp_format_ctype( "LATPOLE", "LATP", "LATP", ftcoord->no, -1, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &poles[1], 1, NULL, NULL );
    }
  }

  if ( cptno == 2 )
  {
    theta = dmi_coord_get_theta( transform );

    if ( theta != 0 )
    { 
      ftp_format_ctype( "CROTA", "TCROT", "TCROT", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &theta, 1, NULL, NULL );
    }
  }
}

/***************************************************************/
/*                                                             */
/* Function:  afp_radesys_equinox_validation                   */
/*                                                             */
/* Description:                                                */
/*       Insure that values for keywords RADESYS and EQUINOX   */
/*       are compatible with each other.                       */
/*                                                             */
/*       Presently, only check for RADESYS = ICRS              */
/*       In future, do more validation of values               */
/*                                                             */
/* Input Parameters:                                           */
/*       dmBlock   *pBlock                                     */
/*       long       colNumber                                  */
/*                                                             */
/* Output Parameters:                                          */
/*                                                             */
/*                                                             */
/* Returns:                                                    */
/*          dmBool status  - either dmTRUE or dmFALSE          */
/*                                                             */
/* Called from:                                                */
/*         afp_scalar_coord_write()                            */
/*                                                             */
/***************************************************************/
dmBool afp_radesys_equinox_validation(dmBlock *pBlock, long colNumber )
{
  dmuKey      *pKeyRadesys = NULL;
  dmuKeyList   keylist;

  dmBool  bIsICRS  = dmFALSE;
  dmBool  bIsGAPPT = dmFALSE;

  afBlock  *pakBlock   = NULL;

  char   *psRadesys = NULL;
  char    sRadesys[CARD_SIZE];
 
//double  equinox_number = 0.0;
//double  EQ1984         = 1984.0;
//double  EQ2000         = 2000.0;

    
  // get keylist
  pakBlock = (afBlock*) dmi_block_get_kb(pBlock);
  keylist  = pakBlock->keylist;

  // get column specific key name RADE#
  sprintf(sRadesys, "%s%d","RADE", (int)colNumber);
  pKeyRadesys = dmu_key_find(keylist, sRadesys);

  if ( !pKeyRadesys )
  {
    // no specific key, get global RADESYS key
    pKeyRadesys = dmu_key_find(keylist, "RADESYS");

    // try RADECSYS key
    if ( !pKeyRadesys )
     pKeyRadesys = dmu_key_find(keylist, "RADECSYS");
  }

  // if no RADESYS, default assumption is "ICRS"

  if ( !pKeyRadesys )
    return dmTRUE;


  // get value of RADE#/RADESYS key
  psRadesys = dmu_key_value_c( pKeyRadesys);

  // is ICRS?
  bIsICRS  = cs_uceq(psRadesys, "ICRS");

  // if not, check for GAPPT
  if ( !bIsICRS )
    bIsGAPPT = cs_uceq(psRadesys, "GAPPT");


  // EQUINOX not applicable for ICRS or GAPPT
  if ( bIsICRS || bIsGAPPT )
    return dmTRUE;

  return dmFALSE;
 }

==> ciao-4.8-src-core.tar.gz: ciao-4.8/src/lib/datamodel/afkernel/afcoord.c <==
/*                                                                
**  Copyright (C) 2003-2009  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/**********************************************************************/
/*                                                                    */
/*  afcoord.c                                                         */
/*                                                                    */
/*                                                                    */
/*                                                                    */
/**********************************************************************/

#include "afkernelP.h"
#include "../ftkernel/ftkernelP.h"

dmBool afp_radesys_equinox_validation(dmBlock *pBlock, long colNumber );

/***********************************************************************/
/*                                                                     */
/*   Function: afp_coord_write_mtype()                                 */
/*                                                                     */
/*   Description:                                                      */
/*              taken from ftkernel                                    */
/*                                                                     */
/*  Input parameters:                                                  */
/*                    dmBlock       *block                             */
/*                    dmDescriptor  *dd                                */
/*                                                                     */
/*  Output parameters:                                                 */
/*                  none                                               */
/*                                                                     */
/*  Returns:                                                           */
/*           void                                                      */
/*                                                                     */
/***********************************************************************/
void afp_coord_write_mtype( dmBlock *block, dmDescriptor *dd )
{
  afBlock *ablock = NULL;

  char  mform[CARD_SIZE];
  char  mtype[CARD_SIZE];
  char  keyname[CARD_SIZE];
  char *value = NULL;
  char *ptr   = NULL;
  char *unit  = NULL; 

  long n     = 0;
  long dim   = 0;
  long cptno = 0;


  dim = dmi_descriptor_get_element_dim( dd );
  cs_copy( dmi_descriptor_get_name( dd ), mtype );  

  ablock = (afBlock*)dmi_block_get_kb( block);

  /* Check if this MTYPE already exists.. */
  n = dmu_keylist_check_index( ablock->keylist, "MTYPE", mtype );
  
  if ( n == 0 )
  {
    /* There are no keywords with this for this system.. create them */
    n = dmu_keyword_max_index( ablock->keylist, "MFORM" );
    ++n;

    /* Write MTYPE keyword */
    sprintf( keyname, "MTYPE%ld", n );
    value = &mtype[0];

    afp_key_write( block, 0, keyname, DT_CHAR, &value, 1,
 		                      NULL, "DM Keyword: Descriptor name." );

    /* Write MFORM keyword */
    sprintf( keyname, "MFORM%ld", n );
    cptno = 1;
    ptr = mform;
    ptr += sprintf( ptr, "%s", dmi_descriptor_get_cpt_name( dd, cptno ) );

    for ( cptno = 2 ; cptno <= dim; cptno++ )
      ptr += sprintf( ptr, ",%s", dmi_descriptor_get_cpt_name( dd, cptno ));

    value = &mform[0];
    unit = dmi_descriptor_get_unit( dd );

    if ( cs_is_blank(unit))
    {
      /* Back compat for regression */
      afp_key_write( block, 0, keyname, DT_CHAR, &value, 1,
		     unit, "DM Keyword: Descriptor value." );
    }
    else
    {
      afp_key_write( block, 0, keyname, DT_CHAR, &value, 1, unit, NULL );
    }
  }
}



/***********************************************************************/
/*                                                                     */
/*  Function: afp_scalar_coord_write()                                 */
/*                                                                     */
/*  Description:   taken from ftkernel                                 */
/*                                                                     */
/*  Input parameters:      dmDescriptor   *dd                          */
/*                         double          crpix                       */
/*                         double          crval                       */
/*                         double          cdelt                       */
/*                         char           *ctype                       */
/*                         long            cptno                       */
/*                         int             is_physical                 */
/*                         int             ok                          */
/*                                                                     */
/*  Output parameters:                                                 */
/*           none                                                      */
/*                                                                     */
/*  Returns:                                                           */
/*           void                                                      */
/*                                                                     */
/***********************************************************************/
void afp_scalar_coord_write( dmDescriptor *dd, double crpix, double crval,
                             double cdelt, char *ctype, long cptno,
                             int is_physical, int ok )
{
  ftCoord *ftcoord = NULL;

  dmBlock          *block     = NULL;
  dmCoordTransform *transform = NULL;
  dmBlockType       type;
  dmBool            bIsICRS   = dmFALSE;

  char   keyname[CARD_SIZE];
  char  *wcstyp = "PHYSICAL";
  char  *unitp  = NULL;
  char  *name   = NULL;
  char  *desc   = NULL;
  char  *unit   = NULL;

  double equinox  = 0.0;
  double poles[2] = { 0.0, 0.0};
  double theta    = 0.0;

  long axisno   = 0;
  long no       = 0;
  long subno    = 0;
  long flag     = 0;
  int  is_image = 0;
  int  axis     = 0;


  if ( !dd )
    return;

  block     = dmi_descriptor_get_block( dd );
  type      = dmi_block_get_raw_type( block );
  ftcoord   = dmi_descriptor_get_kdd( dd );
  transform = dmi_descriptor_get_aux( dd );
  subno     = dmi_coord_get_transform_no( dd );
  no       = dmi_coord_get_scalar_col_no( dd );
  axisno   = dmi_descriptor_get_group_axis_no( dd );
  flag     = dmi_descriptor_get_flag( dd );

  if ( !is_physical )
    is_physical = ( flag & DM_FLAG_PHYS ) || ( ftcoord && ftcoord->sub == 'P' );

  is_image = ( type == dmIMAGE );

  if ( !ftcoord )
  {
    ftcoord = ftp_coord_alloc();
    dmi_descriptor_set_kdd( dd, (void*)ftcoord );

    if ( is_physical )
      ftcoord->sub = 'P';
    else if ( subno > 1 )
      ftcoord->sub =  'A' + subno - 2;
    else
      ftcoord->sub = ' ';

    ftcoord->axis = axisno;

    if ( is_image && axisno > 0 )
      ftcoord->no = 1;
    else
      ftcoord->no = no;
  }

  axis = ( ftcoord && ftcoord->axis > 0 );

  ftp_format_ctype( "CTYPE", "TCTYP", "TCTYP", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );

  desc  = dmi_descriptor_get_desc( dd );
  unit  = dmi_descriptor_get_unit( dd ); 
  unitp = unit;

  if ( dm_global & DM_GLOBAL_OLD )
    unitp = NULL;

  afp_key_write( block, 0, keyname, DT_CHAR, &ctype, 1, unitp, desc );

  ftp_format_ctype( "CRVAL", "TCRVL", "TCRVL", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
  afp_key_write( block, 0, keyname, DT_REAL8, &crval, 1, NULL, NULL );
  ftp_format_ctype( "CRPIX", "TCRPX", "TCRPX", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
  afp_key_write( block, 0, keyname, DT_REAL8, &crpix, 1, NULL, NULL );
  ftp_format_ctype( "CDELT", "TCDLT", "TCDLT", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
  afp_key_write( block, 0, keyname, DT_REAL8, &cdelt, 1, NULL, NULL );

  if ( !ok )
  {
    // FITS Standard Compliance  July 2010
    //  Use TCNA on output instead of TCNAM

    ftp_format_ctype( "CNAME", "TCNA", "TCNA", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );

    name = dmi_descriptor_get_name( dd );
    afp_key_write( block, 0, keyname, DT_CHAR, &name, 1, NULL, NULL );
  }

  if ( is_physical && is_image )
  {
    ftp_format_ctype( "WCSTY", "WCSTY", "WCSTY", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
    afp_key_write( block, 0, keyname, DT_CHAR, &wcstyp, 1, NULL, NULL );
  }

  if ( unit && *unit )
  {
    ftp_format_ctype( "CUNIT", "TCUNI", "TCUNI", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
    ok = ! ( ( dm_global & DM_GLOBAL_OLD ) && is_physical && cs_eq( unit, "pixel" ));

    if ( ok )
      afp_key_write( block, 0, keyname, DT_CHAR, &unit, 1, NULL, NULL );
  }

  /* IRAF and DS9 compatibility */
  if ( is_image && is_physical && cdelt != 0.0 )
  {
    double ltv = 0.0;
    double ltm = 0.0;

    ltv = crpix - crval / cdelt;
    ltm = 1.0 / cdelt;

    sprintf( keyname, "LTV%ld", ftcoord->axis );  
    afp_key_write( block, 0, keyname, DT_REAL8, &ltv, 1, NULL, NULL );  
    sprintf( keyname, "LTM%ld_%ld", ftcoord->axis, ftcoord->axis );  
    afp_key_write( block, 0, keyname, DT_REAL8, &ltm, 1, NULL, NULL );  
  }


  if ( cptno == 1 )
  {
    equinox = dmi_transform_get_equinox( transform );

    // Code Change  FITS Standards  September 15, 2010
    // Need to check value of RADESYS in order to verify EQUINOX valaue

    bIsICRS = afp_radesys_equinox_validation( block, ftcoord->no );


    if (  !bIsICRS && equinox != 0 )
    {
      ftp_format_ctype( "EQUINOX", "EQUI", "EQUI", ftcoord->no, -1, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &equinox, 1, NULL, NULL );
    }

    dmi_transform_get_poles( transform, poles );

    if ( poles[0] != 0.0 )
    {
      ftp_format_ctype( "LONGPOLE", "LONP", "LONP", ftcoord->no, -1, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &poles[0], 1, NULL, NULL );
    }

    if ( poles[1] != 0.0 )
    {
      ftp_format_ctype( "LATPOLE", "LATP", "LATP", ftcoord->no, -1, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &poles[1], 1, NULL, NULL );
    }
  }

  if ( cptno == 2 )
  {
    theta = dmi_transform_get_theta( transform );

    if ( theta != 0 )
    { 
      ftp_format_ctype( "CROTA", "TCROT", "TCROT", ftcoord->no, ftcoord->axis, ftcoord->sub, is_image, axis, keyname );
      afp_key_write( block, 0, keyname, DT_REAL8, &theta, 1, NULL, NULL );
    }
  }
}

/***************************************************************/
/*                                                             */
/* Function:  afp_radesys_equinox_validation                   */
/*                                                             */
/* Description:                                                */
/*       Insure that values for keywords RADESYS and EQUINOX   */
/*       are compatible with each other.                       */
/*                                                             */
/*       Presently, only check for RADESYS = ICRS              */
/*       In future, do more validation of values               */
/*                                                             */
/* Input Parameters:                                           */
/*       dmBlock   *pBlock                                     */
/*       long       colNumber                                  */
/*                                                             */
/* Output Parameters:                                          */
/*                                                             */
/*                                                             */
/* Returns:                                                    */
/*          dmBool status  - either dmTRUE or dmFALSE          */
/*                                                             */
/* Called from:                                                */
/*         afp_scalar_coord_write()                            */
/*                                                             */
/***************************************************************/
dmBool afp_radesys_equinox_validation(dmBlock *pBlock, long colNumber )
{
  dmuKey      *pKeyRadesys = NULL;
  dmuKeyList   keylist;

  dmBool  bIsICRS  = dmFALSE;
  dmBool  bIsGAPPT = dmFALSE;

  afBlock  *pakBlock   = NULL;

  char   *psRadesys = NULL;
  char    sRadesys[CARD_SIZE];
 
//double  equinox_number = 0.0;
//double  EQ1984         = 1984.0;
//double  EQ2000         = 2000.0;

    
  // get keylist
  pakBlock = (afBlock*) dmi_block_get_kb(pBlock);
  keylist  = pakBlock->keylist;

  // get column specific key name RADE#
  sprintf(sRadesys, "%s%d","RADE", (int)colNumber);
  pKeyRadesys = dmu_key_find(keylist, sRadesys);

  if ( !pKeyRadesys )
  {
    // no specific key, get global RADESYS key
    pKeyRadesys = dmu_key_find(keylist, "RADESYS");

    // try RADECSYS key
    if ( !pKeyRadesys )
     pKeyRadesys = dmu_key_find(keylist, "RADECSYS");
  }

  // if no RADESYS, default assumption is "ICRS"

  if ( !pKeyRadesys )
    return dmTRUE;


  // get value of RADE#/RADESYS key
  psRadesys = dmu_key_value_c( pKeyRadesys);

  // is ICRS?
  bIsICRS  = cs_uceq(psRadesys, "ICRS");

  // if not, check for GAPPT
  if ( !bIsICRS )
    bIsGAPPT = cs_uceq(psRadesys, "GAPPT");


  // EQUINOX not applicable for ICRS or GAPPT
  if ( bIsICRS || bIsGAPPT )
    return dmTRUE;

  return dmFALSE;
 }
